@@ -36,8 +36,17 @@ api_bp = Blueprint('api_base_bp', __name__,
                                  static_folder='static',
                                  url_prefix='/api/v1')
 
+merchant_api_bp = Blueprint('merchant_api_bp', __name__,
+                                 template_folder='templates',
+                                 static_folder='static',
+                                 url_prefix='/api/v1/merchant')
+
 api = Api(api_bp)
 
+merchant_api = Api(merchant_api_bp)
+
+
+
 @auth.verify_password
 def verify_user_auth(username, password):
     if not (username and password):
@@ -192,10 +201,11 @@ class CheckAuthTokenResource(SecureAPIResource):
         return 'Ping'             
     
      
-api.add_resource(APIResource,            '/')
+api.add_resource(APIResource,                   '/')
 api.add_resource(APIVersionResource,            '/version')
-api.add_resource(AuthenticateAPIResource,       '/auth')
-api.add_resource(CheckAuthTokenResource,       '/auth-check')    
+api.add_resource(CheckAuthTokenResource,        '/auth-check')
+
+merchant_api.add_resource(AuthenticateAPIResource,       '/auth')    
     
         
         
