@@ -19,6 +19,12 @@ from trexapi.conf import APPLICATION_NAME, APPLICATION_BASE_URL
 from trexmail.email_helper import trigger_send_email
 from trexmodel.models.datastore.merchant_models import MerchantAcct, Outlet
 from trexmodel.models.datastore.customer_models import Customer
+from trexmodel.models.datastore.reward_models import CustomerEntitledVoucher,\
+    CustomerPointReward, CustomerEntitledTierRewardSummary
+from trexapi.utils.api_helpers import generate_user_auth_token
+from trexapi.decorators.api_decorators import user_auth_token_required
+from trexadmin.libs.decorators import elapsed_time_trace
+from flask.json import jsonify
 
 user_api_bp = Blueprint('user_api_bp', __name__,
                                  template_folder='templates',
@@ -47,19 +53,47 @@ def user_register():
                 email           = register_user_form.email.data
                 name            = register_user_form.name.data
                 mobile_phone    = register_user_form.mobile_phone.data
+                birth_date      = register_user_form.birth_date.data
+                gender          = register_user_form.gender.data
                 password        = register_user_form.password.data
                 
+                birth_date = datetime.strptime(birth_date, '%d-%m-%Y')
+                
+                logger.debug('email=%s', email)
+                logger.debug('name=%s', name)
+                logger.debug('mobile_phone=%s', mobile_phone)
+                logger.debug('birth_date=%s', birth_date)
+                logger.debug('gender=%s', gender)
+                logger.debug('password=%s', password)
+                
                 checking_registered_user_acct = User.get_by_email(email)
                 if checking_registered_user_acct is None:
                     if is_not_empty(mobile_phone):
                         checking_registered_user_acct = User.get_by_mobile_phone(mobile_phone)
                         if checking_registered_user_acct is None:
-                            registered_user_acct = User.create(name=name, email=email, mobile_phone=mobile_phone, password=password, create_email_vc=True, create_mobile_phone_vc=True)
+                            registered_user_acct = User.create(name=name, 
+                                                               email=email, 
+                                                               mobile_phone=mobile_phone, 
+                                                               birth_date=birth_date,
+                                                               gender=gender,
+                                                               password=password, 
+                                                               create_email_vc=True, 
+                                                               create_mobile_phone_vc=True
+                                                               )
                             logger.debug('new registered_user_acct=%s', registered_user_acct)
                         else:
-                            return create_rest_message('Mobile Phone have been taken', status_code=StatusCode.BAD_REQUEST)
+                            if checking_registered_user_acct.is_mobile_phone_verified==True:
+                                return create_rest_message('Mobile Phone have been taken', status_code=StatusCode.BAD_REQUEST)
                     else:
-                        registered_user_acct = User.create(name=name, email=email, mobile_phone=mobile_phone, password=password, create_email_vc=True, create_mobile_phone_vc=True)
+                        registered_user_acct = User.create(name=name, 
+                                                               email=email, 
+                                                               birth_date=birth_date,
+                                                               gender=gender,
+                                                               password=password, 
+                                                               create_email_vc=True, 
+                                                               create_mobile_phone_vc=True
+                                                               )
+                        
                         logger.debug('new registered_user_acct=%s', registered_user_acct)
                         
                 else:
@@ -69,10 +103,11 @@ def user_register():
             if registered_user_acct is not None:
                 
                 send_email_verification_code_email(registered_user_acct)
-                
+                (expiry_datetime, token)    = generate_user_auth_token(registered_user_acct.user_id, registered_user_acct.reference_code)
                 
                 return create_rest_message(status_code=StatusCode.OK, 
-                                           reference_code=registered_user_acct.reference_code,
+                                           auth_token                           = token,
+                                           reference_code                       = registered_user_acct.reference_code,
                                            email_vc_expiry_datetime             = str(registered_user_acct.email_vc_expiry_datetime),
                                            mobile_phone_vc_expiry_datetime      = str(registered_user_acct.mobile_phone_vc_expiry_datetime),
                                            )
@@ -108,13 +143,16 @@ def auth_user():
             logger.debug('auth_user: found user account by email=%s', email)    
             logger.debug('auth_user: found user account by password=%s', password)
             if user_acct.is_valid_password(password):
+                (expiry_datetime, token)    = generate_user_auth_token(user_acct.user_id, user_acct.reference_code)
                 return create_rest_message(status_code=StatusCode.OK, 
+                                           auth_token                           = token,
                                            reference_code                       = user_acct.reference_code, 
                                            name                                 = user_acct.name, 
                                            is_email_verified                    = user_acct.is_email_verified, 
                                            is_mobile_phone_verified             = user_acct.is_mobile_phone_verified,
                                            email_vc_expiry_datetime             = str(user_acct.email_vc_expiry_datetime),
                                            mobile_phone_vc_expiry_datetime      = str(user_acct.mobile_phone_vc_expiry_datetime),
+                                           
                                            )
             
             else:
@@ -219,27 +257,24 @@ def register_user_as_customer():
             is_email_used           = False
             is_mobile_phone_used    = False
             outlet_key              = request.headers.get('x-outlet-key')
-            merchant_act_key        = request.headers.get('x-merchant-acct-key')
+            merchant_act_key        = None
+            
             merchant_acct           = None
             
             with db_client.context():
-                merchant_acct       = MerchantAcct.fetch(merchant_act_key)
-                outlet              = Outlet.fetch(outlet_key)
-                
-                    
-                if merchant_acct and outlet:
+                existing_user_acct  = User.get_by_reference_code(reference_code)
+                if existing_user_acct:
+                    outlet              = Outlet.fetch(outlet_key)
                         
-                    logger.debug('merchant_acct.key_in_str=%s', merchant_acct.key_in_str)
-                    logger.debug('outlet.merchant_acct_key=%s', outlet.merchant_acct_key)
-                    
-                    
-                    if merchant_acct.key_in_str == outlet.merchant_acct_key: 
+                    if outlet:
+                        merchant_acct       = outlet.merchant_acct_entity
+                        merchant_act_key    = outlet.merchant_acct_key  
                         logger.debug('Valid granted outlet key for merchant acct')
                         
-                        created_customer = Customer.get_by_reference_code(reference_code)
+                        created_customer = Customer.get_by_reference_code(reference_code, merchant_acct)
                          
                         if created_customer is None:
-                            existing_user_acct  = User.get_by_reference_code(reference_code)
+                            
                             
                             email           = existing_user_acct.email
                             mobile_phone    = existing_user_acct.mobile_phone
@@ -262,27 +297,32 @@ def register_user_as_customer():
                             
                             if is_email_used == False and is_mobile_phone_used == False:
                             
-                                created_customer = Customer.create_from_user(merchant_acct, outlet, existing_user_acct, merchant_reference_code=merchant_reference_code)
+                                created_customer = Customer.create_from_user(outlet, existing_user_acct, merchant_reference_code=merchant_reference_code)
                         
+                            
                         logger.debug('created_customer=%s', created_customer)
+                        
                     else:
-                        logger.warn('Invalid granted outlet key')
-                else:
-                    logger.warn('Invalid granted outlet key or merchant account id')
+                        logger.warn('Invalid granted outlet key or merchant account id')
                 
                 if created_customer:
                     
+                    
+                    
                     response_data = {
                                     'customer_key'              : created_customer.key_in_str,
                                     'registered_datetime'       : created_customer.registered_datetime.strftime("%d-%m-%Y %H:%M:%S"),
                                     'merchant_reference_code'   : created_customer.merchant_reference_code,
                                     'reference_code'            : created_customer.reference_code,
                                     'merchant_account_key'      : merchant_act_key,
-                                    'company_name'              : merchant_acct.name,
+                                    'company_name'              : merchant_acct.company_name,
                                     'outlet_key'                : outlet_key,  
+                                    #'user_details'              : user_details_dict(existing_user_acct),
                                     }
                     
-                    logger.debug('response_data=%s', response_data)
+                    response_data.update(user_details_dict(existing_user_acct))
+                    
+                    logger.debug('register_user_as_customer debug: response_data=%s', response_data)
                     
                     return create_rest_message(status_code=StatusCode.OK, **response_data)
                     
@@ -303,6 +343,158 @@ def register_user_as_customer():
         logger.error('Fail to register customer due to %s', get_tracelog())
         
         return create_rest_message(status_code=StatusCode.BAD_REQUEST)
+
+def user_details_dict(user_acct):
+    is_email_verified           = user_acct.is_email_verified
+    is_mobile_phone_verified    = user_acct.is_mobile_phone_verified
+    
+    logger.debug('user_details_dict debug: is_email_verified=%s', is_email_verified)
+    logger.debug('user_details_dict debug: is_mobile_phone_verified=%s', is_mobile_phone_verified)
+    
+    email_vc_expiry_datetime            = None
+    mobile_phone_vc_expiry_datetime     = None
+    
+    email_verified_datetime             = None
+    mobile_phone_verified_datetime      = None
+    
+    if is_email_verified == False:
+        #vg_generated_datetime = user_acct.vg_generated_datetime.strftime(user_acct.vg_generated_datetime, '%d/%m/%Y, %H:%M:%S')
+        email_vc_expiry_datetime = str(user_acct.email_vc_expiry_datetime)
+    else:
+        email_verified_datetime = str(user_acct.email_verified_datetime)
+            
+    if is_mobile_phone_verified == False:
+        #vg_generated_datetime = user_acct.vg_generated_datetime.strftime(user_acct.vg_generated_datetime, '%d/%m/%Y, %H:%M:%S')
+        mobile_phone_vc_expiry_datetime = str(user_acct.mobile_phone_vc_expiry_datetime)
+    else:
+        mobile_phone_verified_datetime = str(user_acct.mobile_phone_verified_datetime)
+        
+    birth_date_str = None
+    if is_not_empty(user_acct.birth_date):
+        birth_date_str = user_acct.birth_date.strftime('%d-%m-%Y')
+         
+    
+    user_details = {
+                       'reference_code'                       : user_acct.reference_code, 
+                       'name'                                 : user_acct.name, 
+                       'email'                                : user_acct.email, 
+                       'gender'                               : user_acct.gender,
+                       'birth_date'                           : birth_date_str,  
+                       'is_email_verified'                    : is_email_verified,
+                       'is_mobile_phone_verified'             : is_mobile_phone_verified,
+                    }
+    
+    if is_not_empty(email_vc_expiry_datetime):
+        user_details['email_vc_expiry_datetime'] = email_vc_expiry_datetime
+        
+    if is_not_empty(mobile_phone_vc_expiry_datetime):
+        user_details['mobile_phone_vc_expiry_datetime'] = mobile_phone_vc_expiry_datetime
+        
+    
+    if is_not_empty(email_verified_datetime):
+        user_details['email_verified_datetime'] = email_verified_datetime
+        
+    if is_not_empty(mobile_phone_verified_datetime):
+        user_details['mobile_phone_verified_datetime'] = mobile_phone_verified_datetime        
+    
+    return user_details
+    
+@user_api_bp.route('/customer/<reference_code>', methods=['GET'])
+def read_user_customer_acct(reference_code):
+    logger.debug('read_user_customer_acct: reference_code=%s', reference_code)
+    
+    try:
+        if is_not_empty(reference_code):
+            logger.debug('customer registration input is valid')
+            db_client = create_db_client(caller_info="register_user_as_customer")
+            
+            customer                = None
+            outlet_key              = request.headers.get('x-outlet-key')
+            merchant_acct           = None
+            existing_user_acct      = None
+            
+            with db_client.context():
+                outlet          = Outlet.fetch(outlet_key)
+                    
+                if merchant_acct and outlet:
+                        
+                    logger.debug('merchant_acct.key_in_str=%s', merchant_acct.key_in_str)
+                    logger.debug('outlet.merchant_acct_key=%s', outlet.merchant_acct_key)
+                    
+                    
+                    customer = Customer.get_by_reference_code(reference_code, merchant_acct)
+                    existing_user_acct = User.get_by_reference_code(reference_code)
+                     
+                else:
+                    logger.warn('Invalid granted outlet key or merchant account id')
+                
+                if customer:
+                    
+                    response_data = {
+                                    'customer_key'              : customer.key_in_str,
+                                    'registered_datetime'       : customer.registered_datetime.strftime("%d-%m-%Y %H:%M:%S"),
+                                    'merchant_reference_code'   : customer.merchant_reference_code,
+                                    'reference_code'            : customer.reference_code,
+                                    'merchant_account_key'      : customer.registered_merchant_acct_key,
+                                    'outlet_key'                : outlet_key,  
+                                    #'user_details'              : user_details_dict(existing_user_acct), 
+                                    }
+                    
+                    response_data.update(user_details_dict(existing_user_acct))
+                    
+                    logger.debug('read_user_customer_acct debug: response_data=%s', response_data)
+                    
+                    return create_rest_message(status_code=StatusCode.OK, **response_data)
+                    
+                else:
+                    return create_rest_message('Customer account not found', status_code=StatusCode.BAD_REQUEST)
+            
+        else:
+            logger.warn('customer registration input is invalid')
+            
+            return create_rest_message("Missing register customer input data", status_code=StatusCode.BAD_REQUEST)
+    except:
+        logger.error('Fail to read customer details due to %s', get_tracelog())
+        
+        return create_rest_message(status_code=StatusCode.BAD_REQUEST)    
+
+@user_api_bp.route('/customer/<customer_key>/reward/summary', methods=['GET'])
+@user_auth_token_required
+@elapsed_time_trace(trace_key="read_customer_reward_summary")
+def read_customer_reward_summary(customer_key):
+    reference_code  = request.headers.get('x-reference-code')
+    logger.debug('reference_code=%s', reference_code)
+    #vouchers_list   = []
+    tier_rewards    = []
+    
+    db_client = create_db_client(caller_info="read_customer_reward_summary")
+    with db_client.context():
+        customer = Customer.fetch(customer_key)
+        if customer:
+            #customer_vouchers_list          = CustomerEntitledVoucher.list_all_by_customer(customer)
+            customer_tier_reward_summary    = CustomerEntitledTierRewardSummary.list_tier_reward_summary_by_customer(customer)
+            CustomerPointReward.list_by_customer(customer)
+            '''
+            if customer_vouchers_list:
+                for v in customer_vouchers_list:
+                    vouchers_list.append(v.to_dict())
+            '''        
+            if customer_tier_reward_summary:
+                for v in customer_tier_reward_summary:
+                    tier_rewards.append(v.to_dict())
+    
+    
+    
+    result = {
+            'reference_code'    : reference_code,
+            #'vouchers'          : vouchers_list,
+            'tier_rewards'      : tier_rewards,
+            'reward_summary'    : customer.reward_summary,
+            'prepaid_summary'   : customer.prepaid_summary,
+            'voucher_summary'   : customer.entitled_voucher_summary,
+            }
+    
+    return jsonify(result)
     
 @user_api_bp.route('/<reference_code>', methods=['GET'])
 def read_user_acct(reference_code):
@@ -457,7 +649,6 @@ def request_reset_password_post():
         return create_rest_message(status_code=StatusCode.BAD_REQUEST)                 
 
 
-
 def send_email_verification_code_email(user_acct):
     message = '''
                 Hi {name},
